/*
 * This is the console executable that makes use of the BullCowGame class
 * This acts as the view in a MVC pattern and is responsible for all user interaction.
 * For game logic see the FBullCowGame class.
*/

#pragma once

#include <iostream>
#include <string>
#include "FBullCowGame.h"


// to make syntax Unreal-like
using FText = std::string;
using int32 = int;


// Function prototypes as outside a class
void PrintIntro();
void PlayGame();
void PrintGameSummary();
FText GetValidGuess();
bool AskToPlayAgain();


FBullCowGame BCGame; // instantiate a new game, which is re-used across plays.


// future ideas: add a menu for difficulty options
// add option to set isogram - maybe like a 2 player mode
// add exit game from middle of game


// entry point for application
int main ()
{
	bool bPlayAgain = false;

	do
	{
		PrintIntro();
		PlayGame();
		bPlayAgain = AskToPlayAgain();
	}
	while (bPlayAgain);

	return 0;		// exit applicationd
}


void PrintIntro()
{
	// introduce the game
	std::cout << "\n\n Welcome to Bulls and Cows - a fun word guessing game!\n";
	std::cout << std::endl;
	std::cout << " ((...))     ((...))     ((...))     ((...)) " << std::endl;
	std::cout << " ( o o )     ( o o )     ( o o )     ( o o ) " << std::endl;
	std::cout << "  \\   /       \\   /       \\   /       \\   /  " << std::endl;
	std::cout << "  ('_')       ('_')       ('_')       ('_')  " << std::endl;
	std::cout << std::endl;
	std::cout << " There is a secret word. The game is to guess the secret word.\n";
	std::cout << " Make a guess by typing on the keyboard and pressing enter.\n";
	std::cout << " A correct letter in the correct place is called a bull,\n";
	std::cout << " while a correct letter in the wrong place is called a cow.\n";
	std::cout << " After every guess, the game tells you how many bulls and cows were in the last guess.\n";
	std::cout << " You have to figure out the secret word in less than " << BCGame.GetMaxTries() << " guesses.\n";
	std::cout << std::endl;
	std::cout << " An isogram is a word with NO repeating letters. \n";
	std::cout << " Can you guess the " << BCGame.GetHiddenWordLength() << " letter isogram?\n";
	std::cout << std::endl;
	return;
}

// plays a single game to completion
void PlayGame()
{
	BCGame.Reset();

	int32 MaxTries = BCGame.GetMaxTries();

	// Loop for number of turns asking for guesses
	while(!BCGame.IsGameWon() && BCGame.GetCurrentTry() <= MaxTries)
	{
		FText Guess = GetValidGuess();
		
		// submit valid guess to game
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);

		std::cout << " Bulls: " << BullCowCount.Bulls << std::endl;
		std::cout << " Cows:  " << BullCowCount.Cows << "\n\n";
	}

	PrintGameSummary();

	return;
}


// loop continually until user enters valid guess
FText GetValidGuess()
{
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	FText Guess = "";
	do
	{
		// pre-amble
		int32 CurrentTry = BCGame.GetCurrentTry();
		std::cout << " Try " << CurrentTry << " of " << BCGame.GetMaxTries() << ". Go ahead, don't be shy. Guess an isogram: ";

		getline(std::cin, Guess);

		Status = BCGame.CheckGuessValidity(Guess);

		switch (Status)
		{
		case EGuessStatus::Not_Isogram:
			std::cout << " Please enter an isogram.\n\n";
			break;

		case EGuessStatus::Not_English_Lower_Case_Letters:
			std::cout << " Please use only lower-case english letters.\n\n";
			break;

		case EGuessStatus::Wrong_Length:
			std::cout << " Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n\n";
			break;

		default:
			// assume guess is valid
			break;
		}

	} while (Status != EGuessStatus::OK);

	return Guess;
}

void PrintGameSummary()
{
	if (BCGame.IsGameWon())
		std::cout << " YOU WIN!\n";

	else
		std::cout << " YOU LOSE.\n";

	return;
}

bool AskToPlayAgain()
{
	std::cout << " Do you want to play again? (y/n) ";
	FText Response = "";
	getline(std::cin, Response);

	return (Response[0] == 'y' || Response[0] == 'Y');
}